from typing import TYPE_CHECKING, Any, TypeVar

from attrs import define as _attrs_define
from attrs import field as _attrs_field

if TYPE_CHECKING:
    from ..models.device_model import DeviceModel
    from ..models.sequencer_action_features import SequencerActionFeatures


T = TypeVar("T", bound="SequencerAction")


@_attrs_define
class SequencerAction:
    """
    Attributes:
        targets (list['DeviceModel']):
        features (SequencerActionFeatures):
    """

    targets: list["DeviceModel"]
    features: "SequencerActionFeatures"
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        targets = []
        for targets_item_data in self.targets:
            targets_item = targets_item_data.to_dict()
            targets.append(targets_item)

        features = self.features.to_dict()

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "targets": targets,
                "features": features,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: dict[str, Any]) -> T:
        from ..models.device_model import DeviceModel
        from ..models.sequencer_action_features import SequencerActionFeatures

        d = src_dict.copy()
        targets = []
        _targets = d.pop("targets")
        for targets_item_data in _targets:
            targets_item = DeviceModel.from_dict(targets_item_data)

            targets.append(targets_item)

        features = SequencerActionFeatures.from_dict(d.pop("features"))

        sequencer_action = cls(
            targets=targets,
            features=features,
        )

        sequencer_action.additional_properties = d
        return sequencer_action

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties
